/*
    Copyright (C) 1999 Paul Barton-Davis 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: shaped_window.cc,v 1.1.1.1 2001/11/24 00:44:35 pbd Exp $
*/

#include <gtkmm.h>
#include <gtkmm2ext/pix.h>
#include <gtkmm2ext/shaped_window.h>

using namespace Gtkmm2ext;

ShapedWindow::ShapedWindow (Pix *pixset, Gtk::Widget *w)

{
	pix = pixset;
	current_pix = 0;
	exposed = false;
	shapeable_parent = w;
}

ShapedWindow::~ShapedWindow ()

{
	finish_pix (pix);
}

void
ShapedWindow::set_current_pix (size_t n)

{
	n = MIN(n,pix->max_pixmap());

	if (n != current_pix) {
		current_pix = n;
		draw_pixmap ();
	}
}

void
ShapedWindow::on_realize ()
{
	gint width, height;
	Gtk::Widget *parent;

	Gtk::DrawingArea::on_realize ();

	mystyle = get_style();

	if (!shapeable_parent) {
		parent = this;
	} else {
		parent = shapeable_parent;
	}
	
	pix->generate (get_window(), mystyle->get_bg(Gtk::STATE_NORMAL));
	
	pix->pixmap (0)->get_size (width, height);
	set_size_request (width, height);

	parent->shape_combine_mask (pix->shape_mask (0), 0, 0);

	exposed = true;
	draw_pixmap ();
}

bool
ShapedWindow::on_expose_event (GdkEventExpose *ev)

{
	if (ev->count == 0) {
		draw_pixmap ();
	}
	return true;
}

void
ShapedWindow::on_state_changed (Gtk::StateType somestate)

{
	current_pix = (size_t) get_state() % pix->n_pixmaps();
	draw_pixmap();
}

void
ShapedWindow::draw_pixmap ()

{
	if (exposed) {

		if (!pix->homogenous()) {
			Gtk::Widget *parent;
			
			if (!shapeable_parent) {
				parent = this;
			} else {
				parent = shapeable_parent;
			}

			parent->shape_combine_mask (pix->shape_mask (current_pix), 0, 0);
		}

		get_window()->draw_drawable (mystyle->get_bg_gc(get_state()),
					   pix->pixmap (current_pix),
					   0, 0, 0, 0,
					   -1, -1);
	}
}
