/*
    Copyright (C) 1999-2007 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: midibyte_edit.cc,v 1.2 2000/05/02 02:47:16 pbd Exp $
*/

#include <string>
#include <cstring>
#include <gtkmm2ext/utils.h>
#include <midi++/types.h>

#include "midibyte_edit.h"

using namespace sigc;
using namespace Gtk;

MidiByteEditor::MidiByteEditor ()
{
	get_vbox()->pack_start (label);
	get_vbox()->pack_start (entry, true, true, 10);

	label.show ();
	entry.show ();

	add_button (Gtk::Stock::OK, RESPONSE_ACCEPT);
	add_button (Gtk::Stock::CLEAR, RESPONSE_YES);
	add_button (Gtk::Stock::CANCEL, RESPONSE_CANCEL);

	set_position (Gtk::WIN_POS_MOUSE);
	set_title ("MIDI byte editor");
	set_modal (true);

	entry.signal_activate().connect (bind (mem_fun (*this,&Dialog::response), RESPONSE_ACCEPT));
}

MIDI::byte *
MidiByteEditor::edit (const std::string &why, size_t *len, MIDI::byte *msg)
{
	MIDI::byte *buf;
	MIDI::byte *newmsg;
	unsigned int text_length;
	unsigned int new_length;

	label.set_text (why);
	
	//Gtkmm2ext::set_size_request_to_display_given_text (entry, why.c_str(), 10, 4);
	entry.set_hex (msg, *len);

	/* show the window, recurse, and wait till we get an exit */

	bool done = false;

	while (!done) {
		int result = run();
		
		switch (result) {
		case RESPONSE_YES:
			/* means clear */
			entry.set_text ("");
			entry.grab_focus ();
			continue;
		case RESPONSE_ACCEPT:
			done = true;
			break;
		default:
			hide ();
			/* leave everything as it is, just return */
			return msg;
			
		}
	}

	hide ();

	/* post process */

	text_length = entry.get_text_length ();
	buf = new MIDI::byte[text_length]; 
	new_length = entry.get_hex (buf, text_length);

	if (*len < new_length) {
		newmsg = new MIDI::byte[new_length];
	} else {
		newmsg = msg;
	}

	*len = new_length;

	if (new_length) {
		memcpy (newmsg, buf, new_length);
	}

	return newmsg;
}	

