/*
    Copyright (C) 1998-2007 Paul Davis
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: patternfile.h,v 1.1.1.1 2000/04/26 05:31:48 pbd Exp $
*/

#ifndef __sw_patternfile_h__
#define __sw_patternfile_h__

#include <fstream>
#include <map>
#include <vector>

#include <glibmm/ustring.h>

class Sequencer;
class Trigger;

class PatternFile {
  public:
	PatternFile (Sequencer&, const Glib::ustring &path);
	PatternFile (Sequencer&);
	~PatternFile();
	
	bool has_changed () { return edited; }
	
	const Glib::ustring &name() { return _name; }
	void set_name (const Glib::ustring &str);

	std::string pattern (const Glib::ustring& name);
	
	int save ();

	int save_trigger (Trigger&);

	int add_pattern (const Glib::ustring& name, const std::string& pattern);
	int remove_pattern (const Glib::ustring& name);
	int replace_pattern (const Glib::ustring& name, const std::string& pattern);
	int rename_pattern (const Glib::ustring& old_name, const Glib::ustring& new_name);
	
	typedef std::map<Glib::ustring,std::string> PatternMap;
	PatternMap patterns;

	static std::vector<std::string *> *known_pattern_files (const std::string &searchpath);
	
  private:
	Sequencer& sequencer;
	std::fstream f;
	Glib::ustring path;
	Glib::ustring  _name;
	Glib::ustring  dir;
	bool edited;

	void reset ();
	int  parse ();
	void close_file ();

	int load ();

	void set_path_from_name ();
	void set_name_from_path (const Glib::ustring &);
};

#endif  /* __sw_patternfile_h__ */
