#ifndef __sw_patternfile_ui_h__
#define __sw_patternfile_ui_h__

#include <gtkmm.h>
#include <gtkmm2ext/selector.h>
#include <gtkmm2ext/prompter.h>
#include <glibmm/ustring.h>

#include "patternfile.h"

class Sequencer;

class PatternFileUI : public Gtk::Window

{
  public:
	PatternFileUI (PatternFile &, Sequencer&);
	~PatternFileUI ();

  private:
	PatternFile &patternFile;
	Sequencer& sequencer;

	struct PatternColumns : Gtk::TreeModel::ColumnRecord {
	    PatternColumns () {
		    add (name);
		    add (pattern);
	    }
	    Gtk::TreeModelColumn<Glib::ustring> name;
	    Gtk::TreeModelColumn<std::string> pattern;
	};

	Gtk::TreeView patternView;
	Glib::RefPtr<Gtk::ListStore> patternModel;
	PatternColumns pattern_columns;

	Gtkmm2ext::Prompter prompter;

	Gtk::Entry  nameEntry;
	Gtk::Label  nameLabel;
	Gtk::VBox   nameBox;

	Gtk::Button addButton;
	Gtk::Button saveButton;
	Gtk::Button closeButton;
	Gtk::Button removeButton;
	Gtk::Button replaceButton;
	Gtk::Button renameButton;

	Gtk::VBox  modifyBox;
	Gtk::HBox  fileBox;
	Gtk::VBox  packer;
	Gtk::HBox  lrBox;
	Gtk::VBox  tbBox;

	Glib::ustring selected_pattern;

	void refill ();

	/* callbacks */

	void pattern_selected ();
	void pattern_chosen ();
	void prompter_done ();
	
	void save ();
	void close ();

	void remove_pattern ();
	void replace_pattern ();
	void rename_pattern ();
	void add_pattern ();

	bool button_press (GdkEventButton*);

	void rename_file ();

	enum PrompterPending {
		Rename,
		Add
	};

	PrompterPending prompter_pending_operation;
	
};

#endif /* __sw_patternfile_ui_h__ */
