#ifndef __sw_sequence_ui_h__
#define __sw_sequence_ui_h__

#include <vector>
#include <gtkmm.h>
#include <pbd/touchable.h>
#include <gtkmm2ext/click_box.h>

#include "volume_controller.h"
#include "mode.h"

class Sequence;
class StepUI;
class Step;
class Trigger;

class SequenceUI : public Touchable, public Gtk::VBox {
  public:
	SequenceUI (Sequence &seq);
	void touch ();

  protected:
	void map_state ();
	void add_track_button (unsigned int track);
	friend class SequencerUI;

  private:
	Sequence &_sequence;
	vector<StepUI *> steps;
	int ignore_state_changes;

	Gtk::ScrolledWindow stepScroller;
	Gtk::HBox step_hpacker;
	
	Gtk::Adjustment volumeAdjustment;
	VolumeController volumeControl;

	Gtk::HSeparator hsep1;
	Gtk::HSeparator hsep2;
	Gtk::Label sequence_label;
	Gtk::HBox labelSeparatorBox;
	Gtk::HBox hpacker;

	Gtk::VBox rightBox;
	Gtk::HBox rightUpperBox;
	Gtk::HBox rightLowerBox;

	Gtk::HBox mode_buttonBox;
	Gtk::VBox modeControlBox;
	Gtk::VBox volumeBox;

	Gtk::VBox midi_selectionBox;
	Gtk::HBox row_and_triggerBox;
	Gtk::VBox gate_and_noteBox;
	Gtk::VBox midi_and_triggerBox;
	
	Gtk::ComboBoxText modeSelector;
	Gtk::ComboBoxText directionSelector;

	Gtk::Label muteLabel;
	Gtk::Label runLabel;
	Gtk::Label skipLabel;
	Gtk::Label copyLabel;
	Gtk::Label pasteLabel;
	Gtk::Label offLabel;
	Gtk::Label addStepLabel;
	Gtk::Label gateBoxLabel;
	Gtk::Label noteBoxLabel;
	Gtk::Label triggerLabel;
	Gtk::Label modefillLabel;

	Gtk::ToggleButton muteButton;
	Gtk::ToggleButton runButton;
	Gtk::ToggleButton skipButton;
	Gtk::ToggleButton copyButton;

	Gtk::ToggleButton  offButton;
	Gtk::Button        pasteButton;
	Gtk::Button        addStepButton;
	Gtk::Button        triggerButton;
	Gtk::Button        modefillButton;

	Gtk::Adjustment startAdjustment;
	Gtkmm2ext::ClickBox   startSpin;
	Gtk::Frame      startFrame;
	Gtk::HBox       startBox;
	Gtk::Label      startBoxLabel;

	Gtk::Adjustment gateAdjustment;
	Gtkmm2ext::ClickBox   gateSpin;
	Gtk::Frame      gateFrame;
	Gtk::Adjustment noteAdjustment;
	Gtkmm2ext::ClickBox   noteSpin;
	Gtk::Frame      noteFrame;

	Gtk::ComboBoxText  controllerSelector;
	Gtk::ComboBoxText   inportSelector;
	Gtk::ComboBoxText   outportSelector;
	Gtk::Adjustment inchanAdjustment;
	Gtk::Adjustment outchanAdjustment;
	Gtk::SpinButton   inchanSelector;
	Gtk::SpinButton   outchanSelector;
	Gtk::Label      inportBoxLabel;
	Gtk::Label      outportBoxLabel;

	Gtk::Menu      *trigger_menu;
	Gtk::Menu      *trigger_list_menu;
	Gtk::Menu      *modefill_menu;

	vector<Gtk::ToggleButton *> trackButtons;

	/* a whole heap of hboxes to keep everything from expanding
	   beyond their natural size.
	*/

	Gtk::HBox stepScrollerBox;
	Gtk::HBox inportBox;
	Gtk::HBox outportBox;
	Gtk::HBox gateBox;
	Gtk::HBox noteBox;
	Gtk::HBox modeBox;
	Gtk::HBox dirBox;
	Gtk::HBox controllerBox;
	Gtk::HBox trackBox;

	static void _add_step (Step &, void *);
	void add_step (Step &);

	static vector<std::string> mode_strings;
	static vector<std::string> direction_strings;
	
	static bool have_descriptive_strings;
	static void get_descriptive_strings ();

	void map_mode ();
	void map_on_off ();
	gint step_scrollbar_clicked (GdkEventButton *);

	void build_modefill_menu ();
	void modefill_click ();

	void build_trigger_menu ();
	bool rebuild_trigger_menu (GdkEventAny *);
	int add_trigger_item (Trigger &);
	int delete_trigger_item (Trigger &);
	void trigger_click ();

	/* callbacks/signal-handlers */

	void trigger_single_mode ();
	void trigger_multiple_mode ();
	void trigger_multiretrig_mode ();
	void trigger_off_mode ();
	void trigger_selected (Trigger *);
	void trigger_dropped ();

	void new_step ();
	void set_sequence_mode();
	void set_sequence_direction ();
	void set_gate_interval ();
	void set_note_duration ();
	void set_start_step ();
	void set_controller_id ();
	void set_input_port ();
	void set_output_port ();
	void set_input_channel ();
	void set_output_channel ();
	void mute ();
	void run ();
	void skip ();
	void copy ();
	void paste ();
	void off ();
	void mode_fill (MusicalMode::Type);
	void track_toggle (unsigned int);
	void copy_status_change (int);
	void recording_start ();
	void recording_end ();

	static void port_name_printer (char buf[32], Gtk::Adjustment &, void *);
};

#endif /* __sw_sequence_ui_h__ */
