#ifndef __sw_sequencer_step_h__
#define __sw_sequencer_step_h__

/*
    Copyright (C) 1998-2007 Paul Davis
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
    $Id: step.h,v 1.3 2000/05/03 15:54:21 pbd Exp $
*/

#include <sys/types.h>

#include <iostream>
#include <sigc++/signal.h>
#include <midi++/controllable.h>

#include "types.h"
#include "sequence.h"
#include "sequencer.h"

class UI;
class MidiPort;

class Step : public MIDI::Controllable {
  public:
	Sequence &sequence() { return _sequence; }

	void check (unsigned int when);
	void execute (unsigned int when);
	void reset ();
	void enable ();
	void disable ();
	void toggle_active ();

	void set (int val, bool in_single_pitch_set = 0);
	void set (MIDI::byte *msg, size_t size);
	void set_current (bool is_current);
	void set_start (bool is_start);
	void set_end (bool is_end);

	bool is_enabled()  const { return enabled; }
	bool is_current()  const { return _is_current; }
	bool is_start()    const { return _is_start; }
	bool is_end()      const { return _is_end; }
	bool is_terminal() const { return _is_start || _is_end; }

	int val () { return num; }
	MIDI::byte *midimsg (size_t *len = 0) {
		if (len) *len = num; 
		return ptr;
	}
	int id () { return _id; }

	void play (ticks_t when, ticks_t duration) {
		on = 1; time_on = when; time_off = when + duration;
	}

	ticks_t get_next_wakeup () const;

	/* MIDI Controllable interface */

	float lower ();
	float upper ();
	float range ();

	sigc::signal0<void> current_change;
	sigc::signal0<void> value_change;
	sigc::signal0<void> terminal_change;
	sigc::signal0<void> active_change;

  protected:
	/* part of MIDI controllable interface we don't want to be public */

	void set_value (float);

  protected:
	friend class StepUI;

  protected:
	friend class Sequence;
	Step (size_t id, Sequence &);

	void dump (std::ostream &);
	void load (std::istream &);
	friend std::ostream &operator<< (std::ostream &ostr, Step &step);
	friend std::istream &operator>> (std::istream &ostr, Step &step);

  private:
	Sequence &_sequence;
	size_t _id;

	/* These two could, in theory, be combined into a single
	   byte with bit flags. However, they are separated like this 
	   to help with "multi-threading" issues: the handler that
	   runs in response to our MIDI timer only modifies on
	   and the rest of the class structure only modifies
	   enabled. If we combine them into a single field, we will
	   very easily and rapidly end up with garbage as the signal
	   handler goes off in the middle of a bit-level manipulation.
	*/

	bool enabled;
	bool on;
	bool _is_current;
	bool _is_start;
	bool _is_end;

	ticks_t time_on; 
	ticks_t time_off; 
	size_t num;
	MIDI::byte *ptr;

	void input_midi_changed ();
	void note_on (ticks_t when, ticks_t duration);
	void note_off ();


};

std::ostream &operator<< (std::ostream &ostr, Step &step);
std::istream &operator>> (std::istream &ostr, Step &step);

#endif /* __sw_sequencer_step_h__ */
