#ifndef __sw_stepui_h__
#define __sw_stepui_h__

#include <gtkmm.h>
#include <gtkmm2ext/popup.h>

class Step;

class StepUI : public Gtk::VBox
{
  public:
	StepUI (Step &s);

  protected:
	void touch ();      /* called on every iteration of the UI */
	void map_state ();  /* called after substantive changes, eg.
			       after a pattern load.
			    */
	
	friend class SequenceUI;

  private:
	Step &step;
	Gtk::Adjustment adjustment;
	Gtk::EventBox  terminalButton;
	Gtk::HBox      terminalHBox;
	Gtk::Entry     valueDisplay;
	Gtk::Frame     valueDisplayFrame;
	Gtkmm2ext::PopUp learning_popup;
	bool          button_release_can_toggle;
	bool          touched;

	bool ui_is_current;
	bool ui_is_terminal;
	bool ui_is_enabled;

	bool draw_terminal (GdkEventExpose*);

	static gint touch_me (void *);
	void queue_touch ();

	/* These 3 functions handle mouse activity on the 3 components
	   of the UI for a step.
	*/

	bool value_display_button_press (GdkEventButton *);
	bool value_display_button_release (GdkEventButton *);
	bool value_display_key_press (GdkEventKey *);
	bool value_display_scroll_event (GdkEventScroll*);
	gint terminal_click (GdkEventButton *);
	gint pointer_entered_step (GdkEventCrossing *);
	gint pointer_left_step (GdkEventCrossing *);
	
	gint focus_in_step (GdkEventFocus*);
	gint focus_out_step (GdkEventFocus*);

	void adjustment_changed ();
	void silently_set_adjustment (int val);

	void terminal_changed ();
	void value_changed ();
	void current_changed ();
	void active_changed ();
	void value_display_changed ();

	/* how we display the conditions of a step */

	void set_terminal_display ();
	void set_value_display ();
	void set_current_active_display ();

	/* display messages about learning external MIDI control */

	void learning_started();
	void learning_stopped();
};	

#endif /* __sw_stepui_h__ */
