/*
    Copyright (C) 1998-2007 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: ui.h,v 1.8 2000/05/05 14:10:05 pbd Exp $
*/

#ifndef __gtk_ui_h__
#define __gtk_ui_h__

#include <vector>
#include <map>

#include <gtkmm2ext/gtk_ui.h>
#include <gtkmm2ext/click_box.h>

#include "sequence_ui.h"
#include "trigger.h"

class Sequence;
class Sequencer;
class PatternFileUI;
class MidiByteEditor;
class TriggerUI;
class Trigger;

class SequencerUI : public Gtkmm2ext::UI

{
  public:
	SequencerUI (int *, char ***, const char *);

	int set_sequencer (Sequencer&);

	static MidiByteEditor *midibyte_editor () { return theMidiEditor; }
	static SequencerUI *instance () { return theSequencerUI; }

	static Glib::RefPtr<Gdk::Pixbuf> knob_pixbuf;

	static vector<std::string> &port_strings() { return _port_strings; }
	static vector<std::string> &controller_strings() { 
		return _controller_strings ;
	}

	static const char *longest_port_string;
	static const char *longest_controller_string;

	void save_keybindings ();

  private:
	enum ClockMode {
		internal,
		external
	};
		
	Sequencer*           _sequencer;
	Gtk::Window           main_window;
	Gtk::VBox             clockBox;
	Gtk::Adjustment       clock_adjustment;
	Gtk::Frame            clock_frame;
	Gtkmm2ext::ClickBox    clock_spinner;
	Gtk::HScale           tempocontrol;
	Gtk::Button           transport_button;
	Gtk::Button           rew_button;
	Gtk::ToggleButton     clock_mode_button;
	Gtk::Label            clock_mode_label;
	sigc::connection      clock_connection;
	vector<string>        clock_modes;
	Gtk::VBox             vpacker;
	Gtk::VBox             sequence_vpacker;
	Gtk::VBox             scroller_vpacker;
	Gtk::ScrolledWindow   sequence_scroller;
	Gtk::HBox             top_row;
	vector<SequenceUI *> sequences;
	ClockMode            clock_mode;

	int tick_pipe[2];
	int setup_tick_pipe ();
	static void tick_pipe_callback (void *arg, int fd, GdkInputCondition);
	void do_tick ();
	void send_tick (unsigned int);
	
	int setup_pixmaps ();
	int setup_windows ();

	Gtk::MenuBar* menubar;
	Gtk::MenuBar* make_menubar ();

	Gtk::Menu *pattern_file_menu;
	Gtk::Menu *get_trigger_submenu ();
	
	void rebuild_trigger_submenu ();
	void map_state();
	void toggle_pause ();
	void play ();
	void stop ();
	void sync ();
	static void _add_sequence (Sequence &s, void *arg);
	void set_clock_mode (ClockMode);
	void clock_changed ();

	Gtk::ComboBoxText clock_selector;

	void clock_mode_changed ();
	void clock_is_internal ();
	void clock_is_external ();

	void edit_trigger (Trigger *);
	void add_trigger ();

	void trigger_deleted (Trigger &);
	void trigger_name_changed (Trigger &, const string &);

	int add_trigger_item (Trigger &);
	int remove_trigger_item (Trigger &);

	void new_sequence ();
	void add_sequence (Sequence &s);

	typedef std::map<Glib::ustring, PatternFileUI *> PatternFileMap;
	PatternFileMap patternFileMap;
	void open_pattern_file ();
	void open_new_pattern_file ();

	static void clock_filter (char buf[32], Gtk::Adjustment &adj, void *);
	static gboolean do_tick (gint8, gpointer);

	static SequencerUI *theSequencerUI;
	static MidiByteEditor *theMidiEditor;

	static vector<std::string> _port_strings;
	static vector<std::string> clock_mode_strings;
	static vector<std::string> _controller_strings;
	static int grab_port_name (const MIDI::Port &, size_t, void *);
	static void get_descriptive_strings ();

	gint close (GdkEventAny *);

	vector<string *> *pattern_files;

	void add_actions ();
};

extern void integer_filter (char buf[32], Gtk::Adjustment &, void *);

#endif /* __gtk_ui_h__ */

